function SWCS_Program
%Requires: Image Processing Toolbox and Statistics and Machine Learning Toolbox

clc, clear, close all

warning('off')

vess_diam = 5; %mm

total_review_time = 1;

prevx = 0;
prevy = 0;
prevz = 0;


firstpass = 1;
contrast_change = 0;
brightness_change = 0;
cross_line_width = .5;
pointer = 'circle';
processing = 0;

fartery = '';
old_fartery = '';


color_bright = 1;
ladcolor = [1 .9 .9] * color_bright;
lcacolor = [.9 1 .9] * color_bright;
lcxcolor = [.9 .9  1] * color_bright;
rcacolor = [1 1 .9] * color_bright;
nocolor = [0.9400 0.9400 0.9400];


markerColor = [1 0 0];
marker = '.';
markerSize = 51;
markerWidth = 1;


resolution = 0;
zthickness = 0;

roipts = [0 0 0];
borderpts = [0 0 0];

totalSWCS = 0;
totalAS = 0;

global phantom_button_100
artery_buttons = cell(4, 1);

all_points = zeros(1, 3);


global image_folder
global meta_folder


graphics = groot;
mps = graphics.MonitorPositions;

figure_size = min(mps(:, end - 1:end)) .* [0.9896, 0.7];


%Initialize GUI
f = figure('Visible', 'on', 'Position', [0, 0, figure_size(1), figure_size(2)], 'MenuBar', 'none', 'WindowState', 'maximized');

backgroundcolor = [0.95 0.95 0.95]; %Background of figure
set(f, 'Name', 'Interactive DICOM 3D Viewer', 'Color', backgroundcolor);
colormap('Gray'); %Dealing only with grayscaled images (*.DCM)
%Variables
typestring = ''; %Records when numbered keys are pressed
threedarray = NaN; %The three dimensional array given by the DICOM files
global curx
global cury
global curz
curx=1; cury=1; curz=1; %The cross sections of the current images
[P Q R] = size(threedarray); %Size of the three dimensional array
axvert = NaN; axhorz = NaN; sagvert = NaN; saghorz = NaN; corvert = NaN; corhorz = NaN; %For zooming functionality
x2 = NaN; y2 = NaN; %Current Mouse position when depressed
xthickness = NaN; ythickness = NaN; zthickness = NaN; %For measurement tool


%Three axes handles
axial = axes('Position', [0.02 0.14 0.4 0.82], 'DataAspectRatio', [1 1 1], 'XTick', [], 'YTick', [], 'Box', 'on', 'Visible', 'off');


horax = line([1 P], [cury, cury], 'Color', 'b', 'LineWidth', 1, 'Visible',  'off');
verax = line([curx, curx], [1 Q], 'Color', 'g', 'LineWidth', 1, 'Visible',  'off');


horax1 = line(axial, [1 P], [cury, cury], 'Color', 'b', 'LineWidth', 1, 'Visible', 'off');
horax2 = line(axial, [1 P], [cury, cury], 'Color', 'b', 'LineWidth', 1, 'Visible', 'off');
verax1 = line(axial, [curx, curx], [1 Q], 'Color', 'g', 'LineWidth', 1, 'Visible', 'off');
verax2 = line(axial, [curx, curx], [1 Q], 'Color', 'g', 'LineWidth', 1, 'Visible', 'off');


sagittal = axes('Position', [0.45 0.325 0.25 0.5], 'DataAspectRatio', [1 1 1], 'XTick', [], 'YTick', [], 'Box', 'on', 'Visible', 'off');


horsa = line(sagittal, [1 Q], [curz, curz], 'Color', 'r', 'LineWidth', 1, 'Visible',  'off');
versa = line(sagittal, [curx, curx], [1 Q], 'Color', 'b', 'LineWidth', 1, 'Visible',  'off');


coronal = axes('Position', [0.73 0.325 0.25 0.5], 'DataAspectRatio', [1 1 1],  'XTick', [], 'YTick', [], 'Box', 'on', 'Visible', 'off');


horco = line(coronal, [1 Q], [curz, curz], 'Color', 'r', 'LineWidth', 1, 'Visible',  'off');
verco = line(coronal, [curx, curx], [1 R], 'Color', 'g', 'LineWidth', 1, 'Visible',  'off');

instructions = axes('Position', [0.02 0.09 0.96 0.05], 'Visible', 'off'); % 'Box', 'on', 'XTick', [], 'YTick', [],


instructions_text = text(instructions, 0, 0.5, ...
    'Please click on the "Open DICOM" button and select a folder with DICOM files.', ...
    'FontSize', 17, ...
    'Visible', 'on');

%uicontrols

buttonWidth = 100;
buttonHeight = 50;
buttonFontSize = 15;


x_shift_large = 30;
x_shift_small = 10;


x_shift_0 = 20;


uicontrol('Style', 'PushButton', 'Position', [x_shift_0 10 buttonWidth buttonHeight], ...
    'String', 'Open DICOM', 'CallBack', @lookfordicom, 'FontSize', buttonFontSize, 'Units', 'normalized');


x_shift_1 = x_shift_0 + buttonWidth + x_shift_large;

zero_phantom_button = uicontrol('Style', 'PushButton', 'Position', [x_shift_1 10 buttonWidth buttonHeight], ...
    'String', '0 mg/mL Phantom', 'CallBack', @setZeroPhantom, 'FontSize', buttonFontSize, 'Units', 'normalized');

phantom_button_100 = uicontrol('Style', 'PushButton', 'Position', [(x_shift_1+(buttonWidth)) 10 buttonWidth buttonHeight], ...
    'String', '100 mg/mL Phantom', 'CallBack', @set100Phantom, 'FontSize', buttonFontSize, 'Units', 'normalized');

no_phantom_button = uicontrol('Style', 'PushButton', 'Position', [(x_shift_1+(buttonWidth*2)) 10 buttonWidth buttonHeight], ...
    'String', 'No Phantom', 'CallBack', @NoPhantom, 'FontSize', buttonFontSize, 'Units', 'normalized');

files = {'LAD', 'LCA', 'LCX', 'RCA'};


for fn = 1:numel(files)
        
    x_shift_2 = x_shift_1 + (buttonWidth) + x_shift_large + (buttonWidth) * (fn - 1);    
    
    artery_buttons{fn} = uicontrol('Style', 'PushButton', 'Position', [(x_shift_2+(buttonWidth*2)) 10 buttonWidth buttonHeight], ...
        'String', files{fn}, 'CallBack', eval(['@set' files{fn}]), 'FontSize', buttonFontSize, 'Units', 'normalized');
end


% Contrast
x_shift_3 = x_shift_2 + buttonWidth + x_shift_large;


sliderWidth = 200;


uicontrol('Style', 'Text', 'Position', [(x_shift_3+(buttonWidth*2)) 10 100 30], ...
    'String', 'Contrast:', 'FontSize', 15, 'BackgroundColor', backgroundcolor, 'Units', 'normalized');
contrast = uicontrol('Style', 'Edit', 'Position', [(x_shift_3+(buttonWidth*2)) + 90 15 40 24], ...
    'String', 0.5, 'FontSize', 13, 'Enable', 'off', 'BackgroundColor', backgroundcolor, 'CallBack', @contrast_edit, 'Units', 'normalized');
contrast_slide = uicontrol('Style', 'Slider', 'Position', [(x_shift_3+(buttonWidth*2)) + 140 10 sliderWidth 25], ...
    'Value', 0.5, 'Enable', 'off', 'CallBack', @cslide_move, 'Units', 'normalized');


% Brightness
push_up = 35;
uicontrol('Style', 'Text', 'Position', [(x_shift_3+(buttonWidth*2)) 10 + push_up 100 30], ...
    'String', 'Brightness:', 'FontSize', 15, 'BackgroundColor', backgroundcolor, 'Units', 'normalized');
brightness = uicontrol('Style', 'Edit', 'Position', [(x_shift_3+(buttonWidth*2)) + 90 15 + push_up 40 24], ...
    'String', 0.5, 'FontSize', 13, 'Enable', 'off', 'BackgroundColor', backgroundcolor, 'CallBack', @brightness_edit, 'Units', 'normalized');
brightness_slide = uicontrol('Style', 'Slider', 'Position', [(x_shift_3+(buttonWidth*2)) + 140 10 + push_up sliderWidth 25], ...
    'Value', 0.5, 'Enable', 'off', 'CallBack', @bslide_move, 'Units', 'normalized');


% SWCS Button
x_shift_4 = x_shift_3 + 140 + sliderWidth + x_shift_large;


SWCS_button = uicontrol('Style', 'PushButton', 'Position', [(x_shift_4+(buttonWidth*2)) 10 buttonWidth buttonHeight], ...
    'String', 'SWCS', 'CallBack', @get_SWCS, 'FontSize', buttonFontSize, 'Units', 'normalized');


% Draw ROI Button
draw_ROI_button = uicontrol('Style', 'PushButton', 'Position', [620 160 buttonWidth buttonHeight], ...
    'String', 'Draw ROI', 'CallBack', @drawROI, 'FontSize', buttonFontSize, 'Units', 'normalized');


%Make the GUI visible
set(f, 'Visible', 'On');

locx = sum(mps(end, [1, 3])) - mps(end, 3)/2 - figure_size(1)/2;
locy = sum(mps(end, [2, 4])) - mps(end, 4)/2 - figure_size(2)/2;

movegui(f, 'center');

art = imread('Lamprocapnos spectabilis.jpeg');

arta = axes('Position', [0.35 0.1 0.3 0.9], 'Visible', 'on'); 
axes(arta);
arth = imshow(art);
0;

    function lookfordicom(src, eventdata) %#ok<INUSD> %Gather the dicom images into a coherent set
        all_points = zeros(1, 3);
        try
            image_folder = uigetdir;            
            
            idx_subject = strfind(image_folder, filesep);
            subject = image_folder(idx_subject(end) + 1:end);            
            
            meta_folder = fullfile(image_folder(1:idx_subject(end - 1)), 'Meta_Data', subject);
                        
            if ~exist(meta_folder, 'dir')
                mkdir(meta_folder)
            end
                        
            set(f, 'Name', ['Subject Study ID: ' subject], 'NumberTitle', 'off')
            
            phantomf_0 = fullfile(meta_folder, '_0phantom.csv');
            if ~exist(phantomf_0, 'file')
                csvwrite(phantomf_0, [0 0 0])
            end               
            
            phantomf_100 = fullfile(meta_folder, '_100phantom.csv');
            if ~exist(phantomf_100, 'file')
                csvwrite(phantomf_100, [0 0 0])
            end
            
            for file_name = 1:numel(files)
                file = fullfile(meta_folder, [files{file_name} '.csv']);
                if ~exist(file, 'file')
                    csvwrite(file, [0 0 0])
                end
            end
            
            for file_name = 1:numel(files)
                borderf = fullfile(meta_folder, ['_' files{file_name} 'borderpts.csv']);
                if ~exist(borderf, 'file')
                    csvwrite(borderf, [0 0 0])
                end
            end
            
            for file_name = 1:numel(files)
                roiptsf = fullfile(meta_folder, ['_' files{file_name} 'roipts.csv']);
                if ~exist(roiptsf, 'file')
                    csvwrite(roiptsf, [0 0 0])
                end
            end
            
        catch %#ok<CTCH>
            return
        end
        set(f, 'Pointer', 'watch');
        pause(0.01);
        threedarray = NaN;
        try
            [threedarray, xthickness, ythickness, zthickness] = gatherImages(image_folder);
        catch %#ok<CTCH>
            instructions_text.String = 'You selected a folder that does not contain DICOM images';
            threedarray = NaN; xthickness = NaN; ythickness = NaN; zthickness = NaN;
            set(f, 'Pointer', 'arrow', 'KeyPressFcn', '', 'WindowScrollWheelFcn', '');
            set([contrast contrast_slide], 'Enable', 'off');
            cd(meta_folder);
            return
        end
               
        [P, Q, R] = size(threedarray);
        axvert = 1:P; sagvert = 1:P; corvert = 1:Q;
        axhorz = 1:Q; saghorz = 1:R; corhorz = 1:R;
        curx = ceil(P/2); cury = ceil(Q/2); curz = ceil(R/2);
       
        set(arth, 'Visible', 'off')
        
        dispCoronal(curx);
        dispSagittal(cury);
        dispAxial(curz);
                
        firstpass = 0;
                
        set(f, 'Pointer', 'arrow');
        set([brightness brightness_slide contrast contrast_slide], 'Enable', 'on');
        %Press Buttons
        set(f, 'WindowButtonMotionFcn', @mouseMove);
        set(f, 'WindowButtonDownFcn', @clickcallback);
        set(f, 'WindowScrollWheelFcn', @(h_obj, evt) keymove(evt.VerticalScrollCount));
        set(f, 'KeyPressFcn', @(h_obj, evt) keymove(evt.Key));
        
        instructions_text.String = 'Please click on one of the Phantom buttons if a phantom is available. Otherwise, click the "No Phantom" button.';
    end


    function mouseMove(object, eventdata)
        
        fig_loc = get(gcf, 'CurrentPoint');

        if fig_loc(1) > f.Position(3) * axial.Position(1) && fig_loc(1) < f.Position(3) * (axial.Position(3) + axial.Position(1)) ...
                && fig_loc(2) > f.Position(4) * axial.Position(2) && fig_loc(2) < f.Position(4) * (axial.Position(4) + axial.Position(2))
            axes(axial)
            set(f, 'Pointer', pointer);
            d = get(gca, 'CurrentPoint');
            x1 = round(d(1, 1));
            y1 = round(d(1, 2)); %Get the anchor point
            
            if x1>0 && x1<length(axhorz) && y1>0 && y1<length(axvert)
                curx = x1;
                cury = y1;
                
                %For speed of program
                num_groot = findall(groot);
                for i = 1:size(num_groot, 1)
                    if strcmp(num_groot(i).Type, 'line') || strcmp(num_groot(i).Type, 'image') || strcmp(num_groot(i).Type, 'scatter')
                        delete(num_groot(i));
                    end
                end
                
                dispAxial(curz)
                dispSagittal(x1)
                dispCoronal(y1)
                
                axes(axial);
                title(gca, ['(X, Y, Z) = (', num2str(x1), ', ', num2str(y1), ', ', num2str(curz), ')'])
            end

        elseif fig_loc(1) > f.Position(3) * sagittal.Position(1) && fig_loc(1) < f.Position(3) * (sagittal.Position(3) + sagittal.Position(1)) ...
                && fig_loc(2) > f.Position(4) * sagittal.Position(2) && fig_loc(2) < f.Position(4) * (sagittal.Position(4) + sagittal.Position(2))
            axes(sagittal)
            set(f, 'Pointer', pointer);
            d = get(gca, 'CurrentPoint');
            x1 = round(d(1, 1));
            y1 = round(d(1, 2)); %Get the anchor point
            if x1>0 && x1<length(sagvert) && y1>0 && y1<length(saghorz)
                cury = x1;
                curz = y1;
                dispSagittal(curx)
                dispCoronal(x1)
                dispAxial(y1)               
                
                axes(sagittal);
                title(gca, ['(X, Y, Z) = (', num2str(curx), ', ', num2str(x1), ', ', num2str(y1), ')'])
            end
            
        elseif fig_loc(1) > f.Position(3) * coronal.Position(1) && fig_loc(1) < f.Position(3) * (coronal.Position(3) + coronal.Position(1)) ...
                && fig_loc(2) > f.Position(4) * coronal.Position(2) && fig_loc(2) < f.Position(4) * (coronal.Position(4) + coronal.Position(2))
            axes(coronal)
            set(f, 'Pointer', pointer);
            d = get(gca, 'CurrentPoint');
            x1 = round(d(1, 1));
            y1 = round(d(1, 2)); %Get the anchor point
            
            if x1>0 && x1<length(corvert) && y1>0 && y1<length(corhorz)
                curx = x1;
                curz = y1;                              
                dispCoronal(cury)
                dispSagittal(x1)
                dispAxial(y1)
  
                axes(coronal);
                title(gca, ['(X, Y, Z) = (', num2str(x1), ', ', num2str(cury), ', ', num2str(y1), ')'])
            end

        else
            if ~processing
                set(f, 'Pointer', 'arrow');
            end
        end

    end
0;

    function displayInfo(src,data,hAx,img)
        pos = ceil(data.CurrentPosition);
        pixval = img(pos(2),pos(1));
        src.Label = mat2str(pixval);
        title(mat2str(pos));
    end

%Callbacks and subfunctions
    function dispIm(x, y, z) %Update all three images
        dispAxial(z);
        dispSagittal(y);
        dispCoronal(x);
    end
    function dispAxial(z) %Update axial image
        set(f, 'CurrentAxes', axial);

        prevz = curz || ~strcmp(fartery, old_fartery);
                
        im = double(squeeze(threedarray(axvert, axhorz, z)));
        
        im_shift = im - min(im(:));
        
        im_norm = im_shift / max(im_shift(:));
        
        cv = get(contrast_slide, 'Value');
        
        im_contrast = imadjust(im_norm, [cv * .25, 1 - .5 * cv]);
        
        bv = get(brightness_slide, 'Value');
        
        im_bright = im_contrast + .6 * bv - .3;

        imshow(im_bright)
        
        axis square
        
        line(axial, [1 P], [1, 1], 'Color', 'r', 'LineWidth', 2)
        line(axial, [1 P], [Q, Q], 'Color', 'r', 'LineWidth', 2)
        line(axial, [1 1], [1, P], 'Color', 'r', 'LineWidth', 2)
        line(axial, [Q Q], [1, P], 'Color', 'r', 'LineWidth', 2)
        
        ps = 8;   %space for whiskers crossing
        if gcf == f
                horax1 = line([1 curx - ps], [cury, cury], 'Color', 'b', 'LineWidth', cross_line_width, 'Visible', 'on');
                horax2 = line([curx + ps P], [cury, cury], 'Color', 'b', 'LineWidth', cross_line_width, 'Visible', 'on');
                verax1 = line([curx, curx], [1 cury - ps], 'Color', 'g', 'LineWidth', cross_line_width, 'Visible', 'on');
                verax2 = line([curx, curx], [cury + ps Q], 'Color', 'g', 'LineWidth', cross_line_width, 'Visible', 'on');
        end
        logic = all_points(:, 3) == z;
        if any(logic)
            hold on
            xy = all_points(logic, :);
            for i = 1:size(xy, 1)              
                center = [xy(i, 2) xy(i, 1)];
                rad = ((vess_diam/2)/xthickness);
                viscircles(gca, center, rad);
            end

            hold off
            
            markerSize = 51;
        end
        
        logic2 = borderpts(:,3) == z;
        if any(logic2)
            hold on
            xy = borderpts(logic2, :);

            sh = scatter(gca, xy(:, 2), xy(:, 1), markerSize, 'Marker', marker, ...
                'LineWidth', markerWidth, 'MarkerEdgeColor', 'none', 'MarkerFaceColor', 'y');
            sh.MarkerFaceAlpha = .5;
            hold on
            xy = [xy; xy(1,:)];
            roi = plot(gca, xy(:,2), xy(:,1), 'y');
            roi.LineWidth = .5;
            markerSize = 51;
        end
    end
    function dispSagittal(y) %Update sagittal image
        set(f, 'CurrentAxes', sagittal);
        prevx = curx;        
        
        im = double(squeeze(threedarray(sagvert, y, saghorz)))';
        
        im_shift = im - min(im(:));
        
        im_norm = im_shift / max(im_shift(:));
        
        cv = get(contrast_slide, 'Value');
        
        im_contrast = imadjust(im_norm, [cv * .25, 1 - .5 * cv]);
        
        bv = get(brightness_slide, 'Value');
        
        im_bright = im_contrast + .6 * bv - .3;
        
        imshow(im_bright)
        
        axis square;
        
        line(sagittal, [1 P], [1, 1], 'Color', 'g', 'LineWidth', 2)
        line(sagittal, [1 P], [R, R], 'Color', 'g', 'LineWidth', 2)
        line(sagittal, [1 1], [1, Q], 'Color', 'g', 'LineWidth', 2)
        line(sagittal, [Q Q], [1, R], 'Color', 'g', 'LineWidth', 2)
        
        horsa = line([1 Q], [curz, curz], 'Color', 'r', 'LineWidth', cross_line_width, 'Visible', 'on');
        versa = line([cury, cury], [1 R], 'Color', 'b', 'LineWidth', cross_line_width, 'Visible', 'on');
        
        logic = all_points(:, 2) == y;
        if any(logic)
            hold on
            xy = all_points(logic, :);

            sh = scatter(gca, xy(:, 1), xy(:, 3), markerSize, 'Marker', marker, ...
                'LineWidth', markerWidth, 'MarkerEdgeColor', 'none', 'MarkerFaceColor', markerColor);
            sh.MarkerFaceAlpha = .5;
            hold off
            
            markerSize = 51;
        end
        
        logic2 = borderpts(:,2) == y;
        if any(logic2)
            hold on
            xy = borderpts(logic2, :);
                sh = scatter(gca, xy(:, 1), xy(:, 3), markerSize, 'Marker', marker, ...
                    'LineWidth', markerWidth, 'MarkerEdgeColor', 'none', 'MarkerFaceColor', 'y');
                sh.MarkerFaceAlpha = .5;
            markerSize = 51;
        end
    end
    function dispCoronal(x) %Update coronal image
        set(f, 'CurrentAxes', coronal);
        prevy = cury;
        
        im = double(squeeze(threedarray(x, corvert, corhorz)))';
        
        im_shift = im - min(im(:));
        
        im_norm = im_shift / max(im_shift(:));
        
        cv = get(contrast_slide, 'Value');
        
        im_contrast = imadjust(im_norm, [cv * .25, 1 - .5 * cv]);
        
        bv = get(brightness_slide, 'Value');
        
        im_bright = im_contrast + .6 * bv - .3;
        
        imshow(im_bright)
        
        axis square;
        
        line([1 Q], [1, 1], 'Color', 'b', 'LineWidth', 2)
        line([1 Q], [R, R], 'Color', 'b', 'LineWidth', 2)
        line([1 1], [1, P], 'Color', 'b', 'LineWidth', 2)
        line([P P], [1, R], 'Color', 'b', 'LineWidth', 2)
        
        horco = line(coronal, [1 Q], [curz, curz], 'Color', 'r', 'LineWidth', cross_line_width, 'Visible', 'on');
        verco = line(coronal, [curx, curx], [1 R], 'Color', 'g', 'LineWidth', cross_line_width, 'Visible', 'on');
        
        logic = all_points(:, 1) == x;
        if any(logic)
            hold on
            xy = all_points(logic, :);

            sh = scatter(gca, xy(:, 2), xy(:, 3), markerSize, 'Marker', marker, ...
                'LineWidth', markerWidth, 'MarkerEdgeColor', 'none', 'MarkerFaceColor', markerColor);
            sh.MarkerFaceAlpha = .5;
            
            hold off
            
            markerSize = 51;
        end
        
        logic2 = borderpts(:,1) == x;
        if any(logic2)
            hold on
            xy = borderpts(logic2, :);
                sh = scatter(gca, xy(:, 2), xy(:, 3), markerSize, 'Marker', marker, ...
                    'LineWidth', markerWidth, 'MarkerEdgeColor', 'none', 'MarkerFaceColor', 'y');
                sh.MarkerFaceAlpha = .5;
            markerSize = 51;
        end
    end

    function clickcallback(obj, evt)
        persistent chk
        if isempty(chk)
            chk = 1;
            pause(0.5); %Add a delay to distinguish single click from a double click
            if chk == 1
                chk = [];
                
                if strcmp(fartery, '')
                    
                    instructions_text.String = 'Please select an artery before selecting points';
                else
                    points_tool
                end
                
            end
        else
            chk = [];
            
            if ~processing
                d = get(gca, 'CurrentPoint');
                x1 = round(d(1, 1));
                y1 = round(d(1, 2)); %Get the anchor point
                
                distance_to_delete = 5;
                
                if (gca == axial)
                    if x1 > 0 && x1 < length(axhorz) && y1 > 0 && y1 < length(axvert)
                        
                        new_point = [y1, x1, curz];
                        
                        euclid2 = sqrt(sum((all_points(:, 1:2) - new_point(1:2)).^2, 2));
                        euclid3 = sqrt(sum((borderpts(:, 1:2) - new_point(1:2)).^2, 2));
                        
                        [distance, to_delete] = min(euclid2);
                        [distance2, to_delete2] = min(euclid3);
                        
                        if distance <= distance_to_delete
                            
                            all_points(to_delete, :) = [];
                            
                            if isempty(all_points)
                                all_points = [0 0 0];
                            end
                            
                            instructions_text.Color = 'red';
                            instructions_text.String = 'Point Deleted';
                            instructions_text.Color = 'black';
                            
                            csvwrite(fullfile(meta_folder, fartery), all_points)
                            
                        elseif distance2 <= distance_to_delete

                            slic = borderpts(to_delete2, 3);
                            
                            borderpts(borderpts(:,3) == slic, :) = [];
                            
                            if isempty(borderpts)
                                borderpts = [0 0 0];
                            end
                            
                            instructions_text.Color = 'red';
                            instructions_text.String = 'Point Deleted';
                            instructions_text.Color = 'black';
                            
                            csvwrite(fullfile(meta_folder, ['_' fartery(1:3) 'borderpts.csv']), borderpts);

                            if any(roipts(:,3) == slic)
                                roipts(roipts(:,3) == slic, :) = [];
                            end
                            
                            if isempty(roipts)
                                roipts = [0 0 0];
                            end
                            
                            csvwrite(fullfile(meta_folder, ['_' fartery(1:3) 'roipts.csv']), roipts);
                        else
                            instructions_text.Color = 'red';
                            instructions_text.String = 'Nothing Nearby to Delete';
                            instructions_text.Color = 'black';
                        end
                        
                        dispAxial(curz)
                        dispSagittal(x1)
                        dispCoronal(y1)
                        axes(axial)
                    end
                elseif (gca == sagittal)
                    if x1>0 && x1<length(sagvert) && y1>0 && y1<length(saghorz)
                        
                        new_point = [x1, curx, y1];
                        
                        euclid2 = sqrt(sum((all_points(:, [1 3]) - new_point([1 3])).^2, 2));
                        euclid3 = sqrt(sum((borderpts(:, [1 3]) - new_point([1 3])).^2, 2));
                        
                        [distance, to_delete] = min(euclid2);
                        [distance2, to_delete2] = min(euclid3);
                        
                        if distance <= distance_to_delete
                            
                            all_points(to_delete, :) = [];
                            
                            if isempty(all_points)
                                all_points = [0 0 0];
                            end
                            
                            instructions_text.Color = 'red';
                            instructions_text.String = 'Point Deleted';
                            instructions_text.Color = 'black';
                            
                            csvwrite(fullfile(meta_folder, fartery), all_points)
                            
                        elseif distance2 <= distance_to_delete
                            
                            borderpts(to_delete2, :) = [];
                            
                            if isempty(borderpts)
                                borderpts = [0 0 0];
                            end
                            
                            instructions_text.Color = 'red';
                            instructions_text.String = 'Point Deleted';
                            instructions_text.Color = 'black';
                            
                            csvwrite(fullfile(meta_folder, ['_' fartery(1:3) 'borderpts.csv']), borderpts);
                            
                        else
                            instructions_text.Color = 'red';
                            instructions_text.String = 'Nothing Nearby to Delete';
                            instructions_text.Color = 'black';
                        end
                        
                        dispSagittal(curx)
                        dispCoronal(x1)
                        dispAxial(y1)
                        
                        
                        axes(sagittal)
                    end
                elseif (gca == coronal)
                    if x1>0 && x1<length(corvert) && y1>0 && y1<length(corhorz)
                        
                        new_point = [cury, x1, y1];
                        
                        euclid2 = sqrt(sum((all_points(:, [2 3]) - new_point([2 3])).^2, 2));
                        euclid3 = sqrt(sum((borderpts(:, [2 3]) - new_point([2 3])).^2, 2));
                        
                        [distance, to_delete] = min(euclid2);
                        [distance2, to_delete2] = min(euclid3);
                        
                        if distance <= distance_to_delete
                            
                            all_points(to_delete, :) = [];
                            
                            if isempty(all_points)
                                all_points = [0 0 0];
                            end
                            
                            instructions_text.Color = 'red';
                            instructions_text.String = 'Point Deleted';
                            instructions_text.Color = 'black';
                            
                            csvwrite(fullfile(meta_folder, fartery), all_points)
                            
                        elseif distance2 <= distance_to_delete
                            
                            borderpts(to_delete2, :) = [];
                            
                            if isempty(borderpts)
                                borderpts = [0 0 0];
                            end
                            
                            instructions_text.Color = 'red';
                            instructions_text.String = 'Point Deleted';
                            instructions_text.Color = 'black';
                            
                            csvwrite(fullfile(meta_folder, ['_' fartery(1:3) 'borderpts.csv']), borderpts);
                            
                        else
                            instructions_text.Color = 'red';
                            instructions_text.String = 'Nothing Nearby to Delete';
                            instructions_text.Color = 'black';
                        end
                        
                        dispCoronal(cury)
                        dispSagittal(x1)
                        dispAxial(y1)
                        axes(coronal)
                    end
                end
            end
        end
    end

    function get_phantom_samples()        
        
        all_points = all_points(end, :);
                        
        processing = 1;
        set(f, 'Pointer', 'watch');
        pause(0.01);
        
        rad = 5;
        rangex = reshape([all_points(end, 1)-1:-1:all_points(end, 1) - rad;all_points(end, 1)+1:1:all_points(end, 1) + rad], 1, []);
        rangey = reshape([all_points(end, 2)-1:-1:all_points(end, 2) - rad;all_points(end, 2)+1:1:all_points(end, 2) + rad], 1, []);
        k = all_points(end, 3);        
        
        for i = rangex
            for j = rangey
                if size(all_points, 1) < 100
                    all_points = [all_points; [i j k]];
                end
            end
        end        

        %Get points from adjacent slices
        slice_under = k-1;
        zleng = size(all_points,1);
        appz = transpose(repelem(slice_under, zleng));  
        xvals = all_points(:,1);
        yvals = all_points(:,2);
        all_points = [all_points; [xvals yvals appz]];
        
        slice_above = k+1;
        appz = transpose(repelem(slice_above, zleng));        
        all_points = [all_points; [xvals yvals appz]]; 
        
        set(f, 'Pointer', pointer);        
        
        processing = 0;
                
        instructions_text.String = 'Move on to the next phantom concentration. When done selecting both phantoms (0 mg/mL and 100 mg/mL), please click on any coronary artery button, and select points. Adjust brightness and contrast as needed.';
                
    end


    function points_tool(src, eventdata) %#ok<INUSD> %Measures the distance between points
        
        if ~processing
            d = get(gca, 'CurrentPoint');
            x1 = round(d(1, 1));
            y1 = round(d(1, 2)); %Get the anchor point
                        
            if (gca == axial)
                if x1 > 0 && x1 < length(axhorz) && y1 > 0 && y1 < length(axvert)
                    
                    new_point = [y1, x1, curz];
                                                            
                    all_points = [all_points; new_point];
                    
                    all_points(sum(all_points, 2) == 0, :) = [];
                    
                    if strcmp(fartery, '_0phantom.csv')
                        instructions_text.String = 'Collecting samples from the 0 mg/mL phantom...';
                        get_phantom_samples()
                    elseif strcmp(fartery, '_100phantom.csv')
                        instructions_text.String = 'Collecting samples from the 100 mg/mL phantom...';
                        get_phantom_samples() 
                    else
                        instructions_text.String = 'When you finish labeling ALL coronaries, please press the "SWCS" button and wait for the analysis to finish.';
                    end                    
                    
                    csvwrite(fullfile(meta_folder, fartery), all_points)

                    dispAxial(curz)
                    dispSagittal(x1)
                    dispCoronal(y1)
                    axes(axial)
                    
                    %Catch missed slices
                    if ~strcmp(fartery, '_0phantom.csv') && ~strcmp(fartery, '_100phantom.csv')
                        allpts = [all_points; borderpts];
                        allpts(sum(allpts, 2) == 0, :) = [];
                        maxallpts = max(allpts(:,3));
                        minallpts = min(allpts(:,3));
                        slices_labeled = unique(allpts(:,3));
                        if ((maxallpts - minallpts) + 1) > size(slices_labeled)
                            opts = struct('WindowStyle', 'modal', 'Interpreter', 'tex');
                            errormsg = errordlg('\fontsize{15} Not all slices labeled. Press \slEnter \rmto continue', 'Error', opts);
                            uiwait(errormsg);
                        end
                    end
                end
            elseif (gca == sagittal)
                if x1>0 && x1<length(sagvert) && y1>0 && y1<length(saghorz)
                    
                    new_point = [x1, curx, y1];
                    
                    all_points = [all_points; new_point];
                    
                    all_points(sum(all_points, 2) == 0, :) = [];
                    
                    if strcmp(fartery, '_0phantom.csv')
                        instructions_text.String = 'Collecting samples from the 0 mg/mL phantom...';
                        get_phantom_samples()
                    elseif strcmp(fartery, '_100phantom.csv')
                        instructions_text.String = 'Collecting samples from the 100 mg/mL phantom...';
                        get_phantom_samples() 
                    else
                        instructions_text.String = 'When you finish labeling ALL coronaries, please press the "SWCS" button and wait for the analysis to finish.';
                    end 
                                        
                    csvwrite(fullfile(meta_folder, fartery), all_points)

                    dispSagittal(curx)
                    dispCoronal(x1)
                    dispAxial(y1)                    
                    axes(sagittal)
                end
            elseif (gca == coronal)
                if x1>0 && x1<length(corvert) && y1>0 && y1<length(corhorz)
                                        
                    new_point = [cury, x1, y1];
                    
                    all_points = [all_points; new_point];
                    
                    all_points(sum(all_points, 2) == 0, :) = [];
                                        
                    if strcmp(fartery, '_0phantom.csv')
                        instructions_text.String = 'Collecting samples from the 0 mg/mL phantom...';
                        get_phantom_samples()
                    elseif strcmp(fartery, '_100phantom.csv')
                        instructions_text.String = 'Collecting samples from the 100 mg/mL phantom...';
                        get_phantom_samples() 
                    else
                        instructions_text.String = 'When you finish labeling ALL coronaries, please press the "SWCS" button and wait for the analysis to finish.';
                    end 
                                        
                    csvwrite(fullfile(meta_folder, fartery), all_points)
                    
                    dispCoronal(cury)
                    dispSagittal(x1)
                    dispAxial(y1)
                    axes(coronal)
                end
            end
            set(f, 'Pointer', 'arrow');
                        
        end
    end

    function setZeroPhantom(object, eventdata)        
        
        instructions_text.String = 'Click the center of the 0 mg/mL quadrant of the phantom (anterior to patient''s chest). Be sure to check the sagittal and coronoal views as well to ensure the selection is being made in the centern of those views as well.';
        old_fartery = fartery;
        fartery = '_0phantom.csv';
        artery_buttons{1}.BackgroundColor = nocolor;
        artery_buttons{2}.BackgroundColor = nocolor;
        artery_buttons{3}.BackgroundColor = nocolor;
        artery_buttons{4}.BackgroundColor = nocolor;
        SWCS_button.BackgroundColor = nocolor;
        draw_ROI_button.BackgroundColor = nocolor;
        zero_phantom_button.BackgroundColor = [.68 .85 .9];
        phantom_button_100.BackgroundColor = nocolor;
        no_phantom_button.BackgroundColor = nocolor;

        marker = 'o';
        markerSize = 15;
        markerWidth = 2;
                
        file = fullfile(meta_folder, fartery);
        if ~exist(file, 'file')
            csvwrite(file, [0 0 0])
        end
                
        all_points = csvread(fullfile(meta_folder, fartery));
        dispIm(curx, cury, curz)
    end

    function set100Phantom(object, eventdata)
                
        instructions_text.String = 'Click the center of the 100 mg/mL quadrant of the phantom (anterior to patient''s chest). Be sure to check the sagittal and coronoal views as well to ensure the selection is being made in the centern of those views as well.';
        old_fartery = fartery;
        fartery = '_100phantom.csv';
        artery_buttons{1}.BackgroundColor = nocolor;
        artery_buttons{2}.BackgroundColor = nocolor;
        artery_buttons{3}.BackgroundColor = nocolor;
        artery_buttons{4}.BackgroundColor = nocolor;
        SWCS_button.BackgroundColor = nocolor;
        draw_ROI_button.BackgroundColor = nocolor;
        zero_phantom_button.BackgroundColor = nocolor;
        phantom_button_100.BackgroundColor = [.68 .85 .9];
        no_phantom_button.BackgroundColor = nocolor;

        marker = 'o';
        markerSize = 15;
        markerWidth = 2;
        
        file = fullfile(meta_folder, fartery);
        if ~exist(file, 'file')
            csvwrite(file, [0 0 0])
        end        
        
        all_points = csvread(fullfile(meta_folder, fartery));
        dispIm(curx, cury, curz)
    end

    function [mu_avg, sig_avg] = NoPhantom(object, eventdata)
        artery_buttons{1}.BackgroundColor = nocolor;
        artery_buttons{2}.BackgroundColor = nocolor;
        artery_buttons{3}.BackgroundColor = nocolor;
        artery_buttons{4}.BackgroundColor = nocolor;
        SWCS_button.BackgroundColor = nocolor;
        draw_ROI_button.BackgroundColor = nocolor;
        zero_phantom_button.BackgroundColor = nocolor;
        phantom_button_100.BackgroundColor = nocolor;
        no_phantom_button.BackgroundColor = nocolor;
        
        %Obtained from 10 random cases
        mu_avg = 98.6139;
        sig_avg = 17.2241;
        
        instructions_text.String = 'Please click on any coronary artery button, and select points. Adjust brightness and contrast as needed.';
    end

    function review_labeled_points()
        file = fullfile(meta_folder, fartery);
        if ~exist(file, 'file')
            csvwrite(file, [0 0 0])
        end
        all_points = csvread(file);
        
        borderf = fullfile(meta_folder, ['_' fartery(1:3) 'borderpts.csv']);
        if ~exist(borderf, 'file')
            csvwrite(borderf, [0 0 0])
        end
        borderpts = csvread(borderf);
        
        roiptsf = fullfile(meta_folder, ['_' fartery(1:3) 'roipts.csv']);
        if ~exist(roiptsf, 'file')
            csvwrite(roiptsf, [0 0 0])
        end
        roipts = csvread(roiptsf);
        
        borders_slices = unique(borderpts(:,3));
        
        one_each = zeros(size(borders_slices,1), size(borderpts,2));
        for i = 1:size(borders_slices,1)
            rows = borderpts(:,3) == borders_slices(i);
            subset = borderpts(rows, :);
            one_each(i,:) = subset(1,:);
        end
        
        one_each(sum(one_each, 2) == 0, :) = [];
        all_points(sum(all_points, 2) == 0, :) = [];
        
        n_borders = size(one_each, 1);
        n_points = size(all_points, 1);
        Ntotal = n_borders + n_points;
        
        AlltoReview = [one_each; all_points];
        AlltoReview = sortrows(AlltoReview, 3);
        
        pause_per_point = total_review_time / Ntotal;
        
        instructions_text.String = sprintf('Place points or ROIs along %s artery', fartery(1:end-4));
        
        for p = 1:Ntotal
            
            if sum(AlltoReview(:)) ~= 0
                
                cury = AlltoReview(p, 1);
                curx = AlltoReview(p, 2);
                curz = AlltoReview(p, 3);
                
                instructions_text.String = sprintf('You have labeled: point %i', p);
                
            end
            
            dispAxial(curz)
            dispSagittal(curx)
            dispCoronal(cury)
            
            pause(pause_per_point)
        end
        
        instructions_text.String = sprintf('Place points or ROIs along %s artery', fartery(1:end-4));
    end


    function setLAD(src, eventdata)
        
        old_fartery = fartery;
        fartery = 'LAD.csv';
        zero_phantom_button.BackgroundColor = nocolor;
        phantom_button_100.BackgroundColor = nocolor;
        SWCS_button.BackgroundColor = nocolor;
        draw_ROI_button.BackgroundColor = nocolor;
        artery_buttons{2}.BackgroundColor = nocolor;
        artery_buttons{3}.BackgroundColor = nocolor;
        artery_buttons{4}.BackgroundColor = nocolor;
        artery_buttons{1}.BackgroundColor = ladcolor;
        no_phantom_button.BackgroundColor = nocolor;

        marker = 'o';
        markerWidth = 1;
        
        review_labeled_points()
        
    end


    function setLCA(src, eventdata)
                
        old_fartery = fartery;
        fartery = 'LCA.csv';
        zero_phantom_button.BackgroundColor = nocolor;
        phantom_button_100.BackgroundColor = nocolor;
        SWCS_button.BackgroundColor = nocolor;
        draw_ROI_button.BackgroundColor = nocolor;
        artery_buttons{1}.BackgroundColor = nocolor;
        artery_buttons{3}.BackgroundColor = nocolor;
        artery_buttons{4}.BackgroundColor = nocolor;
        artery_buttons{2}.BackgroundColor = lcacolor;
        no_phantom_button.BackgroundColor = nocolor;        

        marker = 'o';
        markerWidth = 1;
                
        review_labeled_points()
        
    end


    function setLCX(src, eventdata)       
        
        old_fartery = fartery;
        fartery = 'LCX.csv';
        zero_phantom_button.BackgroundColor = nocolor;
        phantom_button_100.BackgroundColor = nocolor;
        SWCS_button.BackgroundColor = nocolor;
        draw_ROI_button.BackgroundColor = nocolor;
        artery_buttons{2}.BackgroundColor = nocolor;
        artery_buttons{1}.BackgroundColor = nocolor;
        artery_buttons{4}.BackgroundColor = nocolor;
        artery_buttons{3}.BackgroundColor = lcxcolor;
        no_phantom_button.BackgroundColor = nocolor;

        marker = 'o';
        markerWidth = 1;
                
        review_labeled_points()
        
    end


    function setRCA(src, eventdata)
                
        old_fartery = fartery;
        fartery = 'RCA.csv';
        zero_phantom_button.BackgroundColor = nocolor;
        phantom_button_100.BackgroundColor = nocolor;
        SWCS_button.BackgroundColor = nocolor;
        draw_ROI_button.BackgroundColor = nocolor;
        artery_buttons{2}.BackgroundColor = nocolor;
        artery_buttons{3}.BackgroundColor = nocolor;
        artery_buttons{1}.BackgroundColor = nocolor;
        artery_buttons{4}.BackgroundColor = rcacolor;
        no_phantom_button.BackgroundColor = nocolor;

        marker = 'o';
        markerWidth = 1;
                
        review_labeled_points()
        
    end

pics = {};
series = [];

    function [threedarray, xthickness, ythickness, zthickness] = gatherImages(image_folder)
        %GATHERIMAGES looks through a folder, gets all DICOM files and assembles
        %them into a viewable 3d format.
        d = sortDirectory(image_folder); %Sort in ascending order of instance number
        
        for i = 1:size(d, 1)
            metadata = dicominfo(d{i});
            series(end+1) = metadata.SeriesNumber;
            diff_series = unique(series);
            each_series_sum = zeros(1, size(diff_series, 2));
            for j = 1:size(diff_series, 2)
                each_series_sum(j) = sum(series == diff_series(j));
            end
            max_series = max(each_series_sum);
        end
        
        for i = 1:size(d, 1)
            if series(i) == diff_series(each_series_sum == max_series)
                pics{end+1} = d{i};
            end
        end
        pics = rot90(pics, 3);
        
        topimage = dicomread(pics{1});
        
        if ~all(size(topimage) == [512 512])
            instructions_text.String = sprintf('WARNING: Axial image size is %i x %i. Image size should be 512 x 512.', size(topimage, 1), size(topimage, 2));
            for mes = 1:21
                instructions_text.Color = 'black';
                pause(.1)
                instructions_text.Color = 'red';
                pause(.1)
            end
            instructions_text.String = 'WARNING: Resizing to 512 x 512.';
            pause(2);
            instructions_text.Color = 'black';
            topimage = imresize(topimage, [512, 512]);
        end
        
        metadata = dicominfo(pics{1});
        [group1, element1] = dicomlookup('PixelSpacing');
        [group2, element2] = dicomlookup('SliceThickness');
        resolution = metadata.(dicomlookup(group1, element1));
        xthickness = resolution(1);
        ythickness = resolution(2);
        zthickness = metadata.(dicomlookup(group2, element2));
        threedarray = zeros(size(topimage, 1), size(topimage, 2), size(pics, 1));
        threedarray(:, :, 1) = topimage;
        for i = 2:size(pics, 1)
            metadata = dicominfo(pics{i});
            nextimage = dicomread(metadata);
            if ~all(size(nextimage) == [512 512])
                nextimage = imresize(nextimage, [512, 512]);
            end
            threedarray(:, :, i) = nextimage;
        end      
        
        threedarray = threedarray*metadata.RescaleSlope + metadata.RescaleIntercept;
        
    end


    function d = sortDirectory(image_folder)

        all_files = dir(image_folder);
        all_files = {all_files.name}';
        match_locations = regexp(all_files, '^CT.|.dcm$|.IMA$');
        dicom_file_idx = ~cellfun(@isempty,match_locations);
        d = fullfile(image_folder, all_files(dicom_file_idx));
        m = size(d, 1);
        [group, element] = dicomlookup('InstanceNumber');
        sdata(m) = struct('imagename', '', 'instance', 0);
        for i = 1:m
            metadata = dicominfo(d{i});
            position = metadata.(dicomlookup(group, element));
            sdata(i) = struct('imagename', d{i}, 'instance', position);
        end
        [unused, order] = sort([sdata(:).instance], 'ascend');
        sorted = sdata(order).';
        for i = 1:m
            d{i} = sorted(i).imagename;
        end
    end


    function keymove(key)
        if strcmp(key, 'downarrow') || sum(key)==-1 %If the uparrow is pressed or the mouse wheel is turned
            if (gca == axial && curz > 0 && curz < R) %And we're not out of bounds
                prevz = curz;
                curz = curz + 1;
                dispAxial(curz);
                dispSagittal(curx);
                dispCoronal(cury);
                set(f, 'CurrentAxes', axial);                
                
                title(gca, ['(X, Y, Z) = (', num2str(curx), ', ', num2str(cury), ', ', num2str(curz), ')'])

            elseif (gca == sagittal && curx > 0 && curx < P)
                prevx = curx;
                curx = curx + 1;
                dispAxial(curz);
                dispSagittal(curx);
                dispCoronal(cury);
                set(f, 'CurrentAxes', sagittal);
                                
                title(gca, ['(X, Y, Z) = (', num2str(curx), ', ', num2str(cury), ', ', num2str(curz), ')'])

            elseif (gca == coronal && cury > 0 && cury < Q)
                prevy = cury;
                cury = cury + 1;
                dispAxial(curz);
                dispSagittal(curx);
                dispCoronal(cury);
                set(f, 'CurrentAxes', coronal);               
                
                title(gca, ['(X, Y, Z) = (', num2str(curx), ', ', num2str(cury), ', ', num2str(curz), ')'])
            end
        elseif strcmp(key, 'uparrow') || sum(key)==1 %If the down arrow or mouse wheel is turned
            if (gca == axial && curz > 1 && curz <= R) %And we're not out of bounds
                prevz = curz;
                curz = curz - 1;
                dispAxial(curz);
                dispSagittal(curx);
                dispCoronal(cury);
                set(f, 'CurrentAxes', axial);
                                
                title(gca, ['(X, Y, Z) = (', num2str(curx), ', ', num2str(cury), ', ', num2str(curz), ')'])

            elseif (gca == sagittal && curx > 1 && curx <= P)
                prevx = curx;
                curx = curx - 1;
                dispAxial(curz);
                dispSagittal(curx);
                dispCoronal(cury);
                set(f, 'CurrentAxes', sagittal);
                                
                title(gca, ['(X, Y, Z) = (', num2str(curx), ', ', num2str(cury), ', ', num2str(curz), ')'])

            elseif (gca == coronal && cury > 1 && cury <= Q)
                prevy = cury;
                cury = cury - 1;
                dispAxial(curz);
                dispSagittal(curx);
                dispCoronal(cury);
                set(f, 'CurrentAxes', coronal);
                                
                title(gca, ['(X, Y, Z) = (', num2str(curx), ', ', num2str(cury), ', ', num2str(curz), ')'])

            end

        else %If the wrong key was pressed act is if no button had ever been pressed
            typestring = '';
        end
    end

% change contrast
    function cslide_move(src, eventdata) %#ok<INUSD> %If the slider moves change the label
        set(contrast, 'String', round(100*get(contrast_slide, 'Value'))/100);
        contrast_change = 1;
        dispIm(cury, curx, curz); %Update all images
        contrast_change = 0;
    end
    function contrast_edit(src, eventdata) %#ok<INUSD> %If a value is typed fix the slider
        value = str2double(get(contrast, 'String'));
        if value>1
            set(contrast_slide, 'Value', 1);
            set(contrast, 'String', num2str(1));
        elseif value<0
            set(contrast_slide, 'Value', 0);
            set(contrast, 'String', num2str(0));
        elseif value>0 && value<1
            set(contrast_slide, 'Value', value);
        else
            set(contrast_slide, 'Value', 0.5);
            set(contrast, 'String', num2str(0.5));
        end
        contrast_change = 1;
        dispIm(cury, curx, curz); %Update all images
        contrast_change = 0;
    end


% change brightness
    function bslide_move(src, eventdata) %#ok<INUSD> %If the slider moves change the label
        set(brightness, 'String', round(100*get(brightness_slide, 'Value'))/100);
        brightness_change = 1;
        dispIm(cury, curx, curz); %Update all images
        brightness_change = 0;
    end
    function brightness_edit(src, eventdata) %#ok<INUSD> %If a value is typed fix the slider
        value = str2double(get(brightness, 'String'));
        if value>1
            set(brightness_slide, 'Value', 1);
            set(brightness, 'String', num2str(1));
        elseif value<0
            set(brightness_slide, 'Value', 0);
            set(brightness, 'String', num2str(0));
        elseif value>0 && value<1
            set(brightness_slide, 'Value', value);
        else
            set(brightness_slide, 'Value', 0.5);
            set(brightness, 'String', num2str(0.5));
        end
        brightness_change = 1;
        dispIm(cury, curx, curz); %Update all images
        brightness_change = 0;
    end

mu = 0;
sig = 0;
shifted_mu = 0;

    function get_SWCS(src, eventdata)
        
        processing = 1;
                
        instructions_text.String = 'Computing Spatially Weighted Calcium Score...';
        
        set(f, 'Pointer', 'watch');
        SWCS_button.BackgroundColor = [255, 204, 0] / 255;
        zero_phantom_button.BackgroundColor = nocolor;
        phantom_button_100.BackgroundColor = nocolor;
        phantom_button_100.BackgroundColor = nocolor;
        artery_buttons{1}.BackgroundColor = nocolor;
        artery_buttons{3}.BackgroundColor = nocolor;
        artery_buttons{4}.BackgroundColor = nocolor;
        artery_buttons{2}.BackgroundColor = nocolor;
        no_phantom_button.BackgroundColor = nocolor;
        
        temp = dir(fullfile(meta_folder, '*.csv'));
        point_files = {temp.name}';
        point_files = point_files(~contains(point_files, '_'));
        
        phfile_0 = fullfile(meta_folder, '_0phantom.csv');                
        phfile_100 = fullfile(meta_folder, '_100phantom.csv');        
        
        temp2 = dir(fullfile(meta_folder, '*.csv'));
        border_files = {temp2.name}';
        border_files = border_files(contains(border_files, 'border'));
        
        temp3 = dir(fullfile(meta_folder, '*.csv'));
        roi_files = {temp3.name}';
        roi_files = roi_files(contains(roi_files, 'roi'));
        
     %Constants
        %Attenuation coefficient of water at 100 kVP in cm^2/g
        muH2O = 0.1707; 
        corr_factor = 0.8463972;
        
        %Volume fraction of calcium hydroxyapatite (CHA) and resin for 100 mg/ml phantom
        CHA_vol_100 = 0.033602151;
        res_vol_100 = 1-CHA_vol_100;
        
     %Correction calculation  
        %Loading in each phantom tube's coordinates
        if size(csvread(phfile_0),1) > 2
            phantom_0 = csvread(phfile_0);
            phantom_0([1 2]) = phantom_0([2 1]); 
            phantom_0_N = size(phantom_0, 1);
            
            if phantom_0_N == 1
                instructions_text.Color = 'red';
                instructions_text.String = '0 mg/mL phantom has not been sampled.';
                pause(2)
                return
            end
            
            phantom_100 = csvread(phfile_100);
            phantom_100([1 2]) = phantom_100([2 1]);            
            phantom_100_N = size(phantom_100, 1);
            
            if phantom_100_N == 1
                instructions_text.Color = 'red';
                instructions_text.String = '100 mg/mL phantom has not been sampled.';
                pause(2)
                return
            end            
            
        %Pulling each phantom tube's raw HU values
            rawHU_0 = zeros(phantom_0_N, 1);           
            for i = 1:phantom_0_N               
                loc1 = phantom_0(i, :);
                rawHU_0(i) = threedarray(loc1(1), loc1(2), loc1(3));                
            end
                                   
            rawHU_100 = zeros(phantom_100_N, 1);           
            for i = 1:phantom_100_N               
                loc3 = phantom_100(i, :);
                rawHU_100(i) = threedarray(loc3(1), loc3(2), loc3(3));                
            end            
            
        %Getting resin's attenuation coefficient
            attcoeff_r = zeros(phantom_0_N, 1);
            for i = 1:phantom_0_N
                currHU1 = rawHU_0(i);
                attcoeff_r(i) = muH2O*((currHU1/1000) + 1);
            end
            resin_attcoeff = mean(attcoeff_r);
            
        %Getting CHA's attenuation coefficient            
            %From 100 mg/mL tube
            attcoeff_CHA_100 = zeros(phantom_100_N, 1);
            for i = 1:phantom_100_N
                currHU3 = rawHU_100(i);
                attcoeff_CHA_100(i) = corr_factor*(((muH2O*((currHU3/1000)+1)) - (res_vol_100*resin_attcoeff))/CHA_vol_100);
            end
            
        %Getting corrected 100 mg/mL phantom HU values
            corrHU_100 = zeros(phantom_100_N, 1);
            for i = 1:phantom_100_N
                corrHU_100(i) = (CHA_vol_100*((attcoeff_CHA_100(i)) + (res_vol_100*muH2O) - muH2O)/muH2O) * 1000;
            end
            
     %Cumulative Density Function (CDF) of the 100 mg/mL corrected phantom HU values
            corrHU_100 = sort(corrHU_100);
            sig = std(corrHU_100);
            mu = mean(corrHU_100);
            shifted_mu = mu + 2*sig;
            
            pcdf = normcdf(linspace(50,300,250), shifted_mu, sig);
            
            fh = figure('Visible', 'off');
            plot(linspace(50,300,250), pcdf);
            xlabel('HU');
            ylabel('Pixel Weight');
            fig_CFD = fullfile(meta_folder, '_phantom_CDF.png');
            saveas(gcf, fig_CFD)
            
        else
            [mu_avg, sig_avg] = NoPhantom;
            sig = sig_avg;
            mu = mu_avg;
            shifted_mu = mu + 2*sig;
            
            pcdf = normcdf(linspace(50,300,250), shifted_mu, sig);
            
            fh = figure('Visible', 'off');
            plot(linspace(50,300,250), pcdf);
            fig_CFD = fullfile(meta_folder, '_phantom_CDF.png');
            saveas(gcf, fig_CFD)
            
            no_phantom_button.BackgroundColor = nocolor;
            phantom_100 = [1;1;1];  %assigned this plain col vector so report function doesn't fail
        end
        
        point_files = point_files(~strcmp(point_files, '_100phantom.csv'));
        
        for pf = 1:numel(point_files)
            % Results
            file = fullfile(meta_folder, point_files{pf});
            borderf = fullfile(meta_folder, border_files{pf});
            roif = fullfile(meta_folder, roi_files{pf});
            
            artery_points = zeros(1, 3);
            real_points = zeros(1, 3);
            
            XYZ = csvread(file);
            xyz = csvread(borderf);
            xyz_roi = csvread(roif);
            
            if all(XYZ == [0 0 0])
                
                processing = 0;
                
                message = sprintf('No point information in file: %s', strrep(file, '_', '\_'));
                
                instructions_text.String = message;
                instructions_text.Color = 'red';
                
                pause(2)
                
            end
            
            if all(xyz == [0 0 0])
                
                processing = 0;
                
                message = sprintf('No ROI information in file: %s', strrep(file, '_', '\_'));
                
                instructions_text.String = message;
                instructions_text.Color = 'red';
                
                pause(2)
                
            end
            
            if sum(all(XYZ == [0 0 0])) == 1 && sum(all(xyz == [0 0 0])) == 1
                continue
            end            
            
            X = XYZ(:, 1);
            Y = XYZ(:, 2);
            Z = XYZ(:, 3);
            
            real_points = XYZ;
            border_points = xyz;
            roi_points = xyz_roi;
            
            xyz_res = diag([xthickness, ythickness, zthickness]);
            
            % Display real and fake points
            figure('Visible', 'off')
            plot3(real_points(:, 1), real_points(:, 2), real_points(:, 3), '*r')
            hold on
            plot3(border_points(:, 1), border_points(:, 2), border_points(:, 3), '*', 'Color', '#EDB120')

            legend('Labeled Points','Labeled ROIs')
            rotate3d
            grid on
            axis equal
            axis tight
            view(-50, 20)
            
            fig_points = replace(file, '.csv', '_points.png');
            saveas(gcf, fig_points)
            
            % Compute SWCS 
            if file(end-7) == '/'
                splitfile = split(file, '/');
            else
                splitfile = split(file, '\');
            end
            path = join(splitfile(1:end-1), filesep);
            path = path{1};
            
            [SWCS, xy_shift, region_points, pixar] = get_score(threedarray, real_points, roi_points);
            
            [AS, over130pts, over130hu, numASslices, ASslices] = get_agat(region_points, pixar);
            
            report_name = fullfile(path, replace(splitfile{end}, '.csv', '_SWCS.pdf'));
            message = sprintf('Generating report: %s...', strrep(report_name, '_', '\_'));
            
            instructions_text.Color = 'black';
            instructions_text.String = message;
            
            report(path, splitfile, threedarray, phantom_100, fig_CFD, over130pts, fig_points, SWCS, AS, over130hu, xy_shift, border_points, real_points, numASslices, ASslices)
            
            message = sprintf('Generating report: %s... finished', strrep(report_name, '_', '\_'));
            
            instructions_text.Color = 'black';
            instructions_text.String = message;
            
            totalSWCS = totalSWCS + SWCS;
            totalAS = totalAS + AS;            
            
        end
        
        instructions_text.Color = 'black';
        instructions_text.String = 'Generating Totals report...';
        
        file1 = fullfile(meta_folder, point_files{1});
        if file1(end-7) == '/'
            splitfile1 = split(file1, '/');
        else
            splitfile1 = split(file1, '\');
        end
        path1 = join(splitfile1(1:end-1), filesep);
        path1 = path1{1};
        
        totals_report(path1, splitfile1, phantom_100, fig_CFD, totalSWCS, totalAS)
        
        instructions_text.Color = 'black';
        instructions_text.String = 'Done Processing.';
                
        processing = 0;
                
        SWCS_button.BackgroundColor = [255, 204, 0] / 255;        
        
        set(f, 'Pointer', 'arrow');        
        
        figure(f)
    end
%%
    function [SWCS, xy_shift, region_points, pixar] = get_score(pixar, points, roipoints)
        
        SWCS = 0;
        
        vess_diam_pxs = vess_diam / xthickness;
        vess_rad_pxs = round(vess_diam_pxs / 2);
        range = - vess_rad_pxs: vess_rad_pxs;
        
        range_rep = repmat(range, vess_rad_pxs * 2 + 1, 1);
        range_repT = range_rep';
        
        shift_mat = round(sqrt(range_repT.^2 + range_rep.^2));
        
        x_shift = range_rep(shift_mat <= vess_rad_pxs);
        y_shift = range_repT(shift_mat <= vess_rad_pxs);
        
        xy_shift = [x_shift, y_shift, zeros(length(y_shift), 1)];
        
        region_points = [0 0 0];
        
        for p = 1:size(points, 1)
            
            point = points(p, :);
            
            region_points = [region_points; point + xy_shift];
            
        end
        
        region_points = [region_points; roipoints];
        
        [~, ia, ~] = unique(region_points,'rows','stable');
        i = true(size(region_points, 1), 1);
        i(ia) = false;
        region_points(i, :) = 0;
        region_points(any(region_points == 0, 2), :) = [];
        
        calcium_points = [0 0 0];
        calcium_HU = 0;

        for i = 1:size(region_points, 1)
            
            loc = region_points(i, :);
            
            Wai = normcdf(pixar(loc(1), loc(2), loc(3)), shifted_mu, sig);
            
            Wai1 = normcdf(pixar(loc(1) - 1, loc(2), loc(3)), shifted_mu, sig);
            Wai2 = normcdf(pixar(loc(1) + 1, loc(2), loc(3)), shifted_mu, sig);
            Wai3 = normcdf(pixar(loc(1), loc(2) - 1, loc(3)), shifted_mu, sig);
            Wai4 = normcdf(pixar(loc(1), loc(2) + 1, loc(3)), shifted_mu, sig);
            
            
            sumWai14 = sum([Wai1 Wai2 Wai3 Wai4]);
            
            voxel_score = Wai * sumWai14;      
            
            if any(ismember(region_points, loc, 'rows')) && voxel_score > 0
                calcium_points = [calcium_points; loc];
                calcium_HU = [calcium_HU; pixar(loc(1), loc(2), loc(3))];
            end
            
            SWCS = SWCS + voxel_score;
        end
        
        calcium_points((calcium_HU == 0),:) = [];        
        calcium_HU(calcium_HU == 0) = [];
        calcium_points(any(calcium_points == 0, 2), :) = [];
        
    end

%%
    function drawROI(src, eventdata)
        tempfig = figure('WindowStyle', 'modal');
        tempfig.Position(3:4) = [950 1000];
        WIPz = curz;
        dispAxial(WIPz);
        axi = gca;
        enableDefaultInteractivity(axi);
        
        instructions_text.Color = 'black';
        instructions_text.String = 'Start drawing ROI by clicking to place points and double-clicking to close the polygon. Use scrollwheel to zoom in and out. Once you''ve locked in an ROI, close the pop-up and continue labeling artery.';
        
        roi = drawpolygon('FaceAlpha',0.05,'Color','yellow','LineWidth',1);
        lock = uicontrol('Position', [20 6 200 40], 'String', 'Lock in', 'Callback', 'uiresume(gcbf)', 'BackgroundColor', [.9 .9 1], 'FontSize', 20);
        redraw = uicontrol('Position', [400 6 220 40], 'String', 'Redraw ROI', 'Callback', @redrawroi, 'BackgroundColor', [1 1 .9], 'FontSize', 20);
        
        if ~ishandle(tempfig)
            delete(lock);
            delete(redraw);
            instructions_text.String = 'When you finish labeling ALL coronaries, please press the "SWCS" button and wait for the analysis to finish.';
        end
        
        disableDefaultInteractivity(axi);
        axi.Interactions = zoomInteraction;
        
        if ~ishandle(tempfig)
            delete(lock);
            delete(redraw);
            instructions_text.String = 'When you finish labeling ALL coronaries, please press the "SWCS" button and wait for the analysis to finish.';
        end
        
        uiwait(gcf);
        
        function redrawroi(src,eventdata)
            delete(roi);
            enableDefaultInteractivity(axi);
            
            roi = drawpolygon('FaceAlpha',0.05,'Color','yellow','LineWidth',1);
            lock = uicontrol('Position', [20 6 200 40], 'String', 'Lock in', 'Callback', 'uiresume(gcbf)', 'BackgroundColor', [.9 .9 1], 'FontSize', 20);
            redraw = uicontrol('Position', [400 6 220 40], 'String', 'Redraw ROI', 'Callback', @redrawroi, 'BackgroundColor', [1 1 .9], 'FontSize', 20);
            
            disableDefaultInteractivity(axi);
            axi.Interactions = zoomInteraction;
            
            if ~ishandle(tempfig)
                delete(lock);
                delete(redraw);
                instructions_text.String = 'When you finish labeling ALL coronaries, please press the "SWCS" button and wait for the analysis to finish.';
            end
            
            uiwait(gcf);
        end
        
        if ~ishandle(tempfig)
            delete(lock);
            delete(redraw);
            instructions_text.String = 'When you finish labeling ALL coronaries, please press the "SWCS" button and wait for the analysis to finish.';
        end
        
        instructions_text.String = 'When you finish labeling ALL coronaries, please press the "SWCS" button and wait for the analysis to finish.';
        
        coords = round(roi.Position);
        
        xv = coords(:,1);
        yv = coords(:,2);
        inpoly = poly2mask(xv,yv,(size(threedarray,2)),(size(threedarray,1)));
        
        coords1 = coords(:,1);
        coords2 = coords(:,2);
        coords = [coords2,coords1];
        
        [row1,col1] = find(inpoly == 1);
        zlength = size(row1,1);
        appendz = transpose(repelem(WIPz, zlength));
        roipts = [roipts;row1 col1 appendz];
        roipts(sum(roipts, 2) == 0, :) = [];
        
        zleng = size(coords,1);
        appz = transpose(repelem(WIPz, zleng));
        coordsz = [coords appz];
        borderpts = [borderpts; coordsz];
        borderpts(sum(borderpts, 2) == 0, :) = [];
        
        csvwrite(fullfile(meta_folder, ['_' fartery(1:3) 'roipts.csv']), roipts);
        
        csvwrite(fullfile(meta_folder, ['_' fartery(1:3) 'borderpts.csv']), borderpts);        
        
        dispAxial(WIPz);
        
        uiwait(tempfig);
        
        %Catch missed slices
        if ~ishandle(tempfig)
            all_points(sum(all_points, 2) == 0, :) = [];
            allpts = [all_points; borderpts];
            allpts(sum(allpts, 2) == 0, :) = [];
            maxallpts = max(allpts(:,3));
            minallpts = min(allpts(:,3));
            slices_labeled = unique(allpts(:,3));
            if ((maxallpts - minallpts) + 1) > size(slices_labeled)
                opts = struct('WindowStyle', 'modal', 'Interpreter', 'tex');
                errormsg = errordlg('\fontsize{15} Not all slices labeled. Press \slEnter \rmto continue', 'Error', opts);
                uiwait(errormsg);
            end
        end
        
        coords = [0 0];
    end

%% Agatston score calculation
    function [AS, over130pts, over130hu, numASslices, ASslices] = get_agat(region_points, pixar)
        
        %Get the spacing between slices (voxel_height) 
        metadata_top = dicominfo(pics{1});
        metadata_bot = dicominfo(pics{end});
        
        [group1, element1] = dicomlookup('ImagePositionPatient');
        slice_orig_top = metadata_top.(dicomlookup(group1, element1));
        slice_orig_bot = metadata_bot.(dicomlookup(group1, element1));
        
        [group2, element2] = dicomlookup('ImageOrientationPatient');
        rotation_v1_v2 = metadata_top.(dicomlookup(group2, element2));
        
        ROT = [rotation_v1_v2(1:3) rotation_v1_v2(4:end) cross(rotation_v1_v2(1:3), rotation_v1_v2(4:end))];
        
        slice_orig_top_bot_R = ROT * [slice_orig_top slice_orig_bot];
        
        z_top = slice_orig_top_bot_R(3, 1);
        z_bot = slice_orig_top_bot_R(3, 2);
        
        z_extent = z_top - z_bot;
        
        voxel_height = z_extent / (numel(pics) - 1);
                
        over130hu = [];
        over130pts = [];  
        
        min_contig = ceil(1/(xthickness*ythickness*voxel_height));  %Minimum number of contiguous voxels needed to count as lesion
        asi_slice = [];
        AS = 0;
        gather = [];
        ASslices = [];
        
        %Gather coordinates and HUs of all selected pixels that have attenuation of >= 130
        for b = 1:size(region_points, 1)
            if pixar(region_points(b,1), region_points(b,2), region_points(b,3)) >= 130
                over130hu(b,:) = pixar(region_points(b,1), region_points(b,2), region_points(b,3));
                over130pts(b,:) = region_points(b,:);                
            end
        end
        
        if ~isempty(over130pts)  %Make sure over130pts (vector of voxel coordinates (XYZ) in this case with HU >= 130) is not empty
            
            over130pts((over130hu == 0),:) = [];
            over130hu(over130hu == 0) = [];
            slices = unique(over130pts(:,3));
            
            for i = 1:size(slices)                 %Loop through each slice in this case with HU >= 130
                clusters = zeros(size(pixar, 1));  %Create 512 x 512 matrix of zeros
                k = 1;
                curslice = slices(i);
                for l = 1:size(over130pts, 1)
                    z = over130pts(l,3);
                    if curslice == z  %Only look at points that are on this slice
                        gather(k,:) = [over130pts(l,1), over130pts(l,2)];  %Gather XY points on this slice
                        k = k + 1;
                    end
                end
                
                if ~isempty(gather)
                    
                    gather(any(gather == 0, 2), :) = [];  %Get rid of rows in gather that are 0,0
                    
                    inx = sub2ind(size(clusters), gather(:,1), gather(:,2));  %Match gather coordinates to clusters and make those elements = 1
                    clusters(inx) = 1;
                    
                    [clustersImage, numberOfClusters] = bwlabel(clusters);    %Create image matrix that assigns indices to each separate cluster
                    
                    goodClusters = [];
                    
                    for h = 1:numberOfClusters                %Loop through each cluster on this slice
                        [r1, c1] = find(clustersImage == h);  %Get XY coordinates within this cluster
                        rc1 = [r1 c1];
                        num_in_cluster = size(rc1, 1);        %Number of pixels in this cluster
                        
                        if num_in_cluster >= min_contig       %If number of pixels in this cluster meets minimum criteria, get the index for this cluster and put in goodClusters
                            goodClusters(end+1) = h;
                        end
                    end
                    
                    if ~isempty(goodClusters)
                        asi = [];
                        ASslices(i) = curslice;
                        for m = 1:size(goodClusters, 2)  %Loop through each cluster that meets minimum criteria
                            [r2, c2] = find(clustersImage == goodClusters(m));  %Get XY coordinates within this "good" cluster
                            rc2 = [r2 c2];
                            
                            HU_in_cluster = [];
                            
                            for v = 1:size(rc2,1)                  %Loop through each coordinate within this good cluster
                                xy = rc2(v, :);
                                for w = 1:size(over130pts, 1)  %Loop through each coordinate within over130pts
                                    xy2 = over130pts(w, 1:2);
                                    if (over130pts(w, 3) == slices(i)) && xy(1) == xy2(1) && xy(2) == xy2(2)  %Compare good cluster coordinate with over130pts coordinate
                                        HU_in_cluster(end+1) = over130hu(w);  %If they match, get the HU for that coordinate
                                        break
                                    end
                                end
                            end
                            
                            maxHU = max(HU_in_cluster);  %Find the max HU in this good cluster
                            
                            if maxHU <= 199
                                df = 1;
                            elseif maxHU >= 200 && maxHU <= 299
                                df = 2;
                            elseif maxHU >= 300 && maxHU <= 399
                                df = 3;
                            elseif maxHU >= 400
                                df = 4;
                            end
                            
                            goodClusterarea = size(HU_in_cluster, 2) * xthickness * ythickness;  %Calculate area of this good cluster
   
                            asi(end+1) = goodClusterarea * df;  %Get Agatston score of this good cluster
                            factor = zthickness / voxel_height; %Create factor to adjust AS by based on interslice distance
                            
                            asi = asi / factor;     %Divide AS of this slice by the factor
                        end
                        asi_slice(end+1) = sum(asi);  %Sum Agatston scores for this slice's good clusters
                    end
                    gather = [];
                end
            end
        else
            clusters = zeros(size(pixar, 1));
        end
        AS = sum(asi_slice);  %Sum Agatston scores for this case to get final Agatston score
        numASslices = size(asi_slice, 2);
    end
%%
    function report(path, splitfile, threedarray, phantom_100, fig_CFD, calcium_points, fig_points, SWCS, AS, calcium_HU, xy_shift, border_points, real_points, numASslices, ASslices)
        
        import mlreportgen.dom.*;
        import mlreportgen.report.*;
        
        report_name = fullfile(path, replace(splitfile{end}, '.csv', '_SWCS'));
        
        rep = Report(report_name, 'pdf');
        
        sec1 = Section('Spatially Weighted Calcium Score (SWCS) Report');
        
        id = Text(sprintf('Subject Project ID:\n %s', splitfile{end - 1}));
        id.Bold = true;
        
        coronary = Text(sprintf('Coronary:\n %s', splitfile{end}(1:end-4)));
        coronary.Bold = true;
        
        score = Text(sprintf('SWCS:\n %i', SWCS));
        score.Bold = true;
        score.Color = 'red';
        
        ag_score = Text(sprintf('Agatston Score:\n %i', AS));
        ag_score.Bold = true;
        ag_score.Color = 'red';
        
        numASslices = num2str(numASslices);
        extent = Text(sprintf('      - Extent of Calcification (AS): %s slices', numASslices));
        extent.Color = 'red';
        extent.WhiteSpace = 'pre-wrap';
        
        add(sec1, id);
        add(sec1, coronary);
        add(sec1, score);
        add(sec1, ag_score);
        add(sec1, extent);   %extent of calcification (# of slices used in AS calculation)
        
        sec2 = Section('Phantom (100 mg/mL) HU Cumulative Density Function (CDF)');
        
        im1 = Image(fig_CFD);
        im1.Style = {ScaleToFit};
        
        add(sec2, im1);
        add(sec1, sec2)
        
        if size(phantom_100,1) < 4
            sec2 = Section('Phantom (100 mg/mL)');
            message = Text('The results in this report are based on phantom data from 10 random cases due to lack of phantom. The results should be interpreted accordingly');
            add(sec2, message);
            add(sec1, sec2);
        end
        
        sec3 = Section('Labeled Coronary Artery Points');
        
        im2 = Image(fig_points);
        im2.Style = {ScaleToFit};
        
        add(sec3, im2);
              
        add(sec1, sec3)               
        
        if size(phantom_100,1) > 4
            sec4 = Section('Phantom Patch');
            from = min(phantom_100);
            to = max(phantom_100);
            
            pad = 30;
            
            if from(1) - pad > 1
                padlowx = pad;
            else
                padlowx =  from(1) - 1;
            end
            patch = threedarray(from(1) - padlowx:to(1) + pad, from(2) - pad:to(2) + pad, from(3));
            
            figure('Visible', 'off')
            imagesc(patch/(max(patch(:))) * 255);
            colormap gray
            
            fig_phan_patch = fullfile(meta_folder, '_phantom_patch.png');
            saveas(gcf, fig_phan_patch)
            
            fig = Figure(gcf);
            add(sec4, fig)
            
            add(sec1, sec4)
        end
        
        msg = Text(sprintf('                                 This slice was included in the Agatston Score calculation'));
        msg.WhiteSpace = 'pre-wrap';
        
        if ~isempty(calcium_HU)
            sec5 = Section('Labeled Points Image Patches');
            
            pad = 50;
            
            calcium_points(any(calcium_points == 0, 2), :) = [];
            
            ca_slices = unique(calcium_points(:,3));
            ca_slices = sort(ca_slices);
            
            ASslices(ASslices == 0) = [];
            ASslices = sort(ASslices);
            
            for j = 1:size(ca_slices,1)
                xys = [];
                if border_points(1:3) ~= 0
                    xys = border_points(border_points(:,3) == ca_slices(j),:);
                end
                if ~isempty(xys)
                    xys(:,3) = [];
                    xys = [xys; xys(1,:)];
                end
                
                xys_ = [];
                if real_points(1:3) ~= 0
                    xys_ = real_points(real_points(:,3) == ca_slices(j),:);
                end               
                if ~isempty(xys_)
                    xys_(:,3) = [];
                end
                
                patch = threedarray(:,:, ca_slices(j));

                get_fig = figure('Visible','off');
                imagesc(patch);
                colormap gray
                hold on
                if ~isempty(xys_)
                    center = [xys_(:, 2) xys_(:, 1)];
                    rad = repelem(((vess_diam/2)/xthickness), size(center,1));
                    viscircles(gca, center, rad);
                end
                hold on
                if ~isempty(xys)
                    plot(gca, xys(:,2), xys(:,1), 'y', 'LineWidth', 2);
                end
                
                fig_patch = fullfile(path, [splitfile{end}(1:end-4) '_patch_' num2str(j) '.png']);
                saveas(gcf, fig_patch)
                
                delete(get_fig)
                
                im2 = Image(fig_patch);
                im2.Style = {ScaleToFit};
                
                add(sec5, im2)
                
                if ismember(ca_slices(j), ASslices)
                    add(sec5, msg);
                end
                
                0;
            end
            
            add(sec1, sec5)
            
        end
        
        add(rep, sec1)
        
        close(rep);
        0;
        
    end

%%
    function totals_report(path, splitfile, phantom_100, fig_CFD, SWCS, AS)
        
        import mlreportgen.dom.*;
        import mlreportgen.report.*;
        
        report_name = fullfile(path, 'Total_SWCS');
        
        rep = Report(report_name, 'pdf');
        
        sec1 = Section('Spatially Weighted Calcium Score (SWCS) Report');
        
        id = Text(sprintf('Subject Project ID:\n %s', splitfile{end - 1}));
        id.Bold = true;       
        
        score = Text(sprintf('Total SWCS:\n %i', SWCS));
        score.Bold = true;
        score.Color = 'red';
        
        ag_score = Text(sprintf('Total Agatston Score:\n %i', AS));
        ag_score.Bold = true;
        ag_score.Color = 'red';
        
        add(sec1, id);
        add(sec1, score);
        add(sec1, ag_score);
        
        sec2 = Section('Phantom (100 mg/mL) HU Cumulative Density Function (CDF)');
        
        im1 = Image(fig_CFD);
        im1.Style = {ScaleToFit};
        
        add(sec2, im1);
        add(sec1, sec2)
        
        if size(phantom_100,1) < 4
            sec2 = Section('Phantom (100 mg/mL)');
            message = Text('The results in this report are based on phantom data from 10 random cases due to lack of phantom. The results should be interpreted accordingly');
            add(sec2, message);
            add(sec1, sec2);
        end
        
        add(rep, sec1)
        
        close(rep);
        0;
        
    end
end